//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include "aiousb.h"
#include "detect.h"
//---------------------------------------------------------------------------

#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------

#pragma pack(push, 1)
typedef struct _TWatchdogConfig
{
    WORD TimeoutDeciseconds;
    BYTE OutMask;
    WORD DIOData;
} TWatchdogConfig, *TPWatchdogConfig;
#pragma pack(pop)
//---------------------------------------------------------------------------

__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormCreate(TObject *Sender)
{
   LevelImage[ 0] = Image1;
   LevelImage[ 1] = Image2;
   LevelImage[ 2] = Image3;
   LevelImage[ 3] = Image4;
   LevelImage[ 4] = Image5;
   LevelImage[ 5] = Image6;
   LevelImage[ 6] = Image7;
   LevelImage[ 7] = Image8;
   LevelImage[ 8] = Image9;
   LevelImage[ 9] = Image10;
   LevelImage[10] = Image11;
   LevelImage[11] = Image12;
   LevelImage[12] = Image13;
   LevelImage[13] = Image14;
   LevelImage[14] = Image15;
   LevelImage[15] = Image16;

   OutButton[0] = OutButtonA;
   OutButton[1] = OutButtonB;

   LowOutputBitmap = new Graphics::TBitmap;
   HighOutputBitmap = new Graphics::TBitmap;
   InputBitmap = new Graphics::TBitmap;

   LevelImages->GetBitmap(0, LowOutputBitmap);
   LevelImages->GetBitmap(1, HighOutputBitmap);
   LevelImages->GetBitmap(2, InputBitmap);

   SetWatchdogButton->Align = alRight;

   DeviceIndex = diOnly;

   unsigned long PID = 0;
   unsigned long Status = QueryDeviceInfo(DeviceIndex, &PID, NULL, NULL, NULL, NULL);
   if ( (Status != ERROR_SUCCESS) || ((PID & 0xFEE0) != 0x8040) )
   {
        TForm2 *DetectForm = new TForm2(NULL);
        if (DetectForm->ShowModal() == mrOk)
        {
            DeviceIndex = DetectForm->DeviceIndex;
            delete DetectForm;
        }
        else
        {
            delete DetectForm;
            exit(0);
        }
   }

   DIOData = 0xFFFF;
   UpdateDIODisplay();

   TimeoutTrack->Position = 100;

   BlurbMemo->Lines->Text =
      "First, use the Digital I/O section to configure a DIO state. Note that this state is not immediately written to the device, instead it's saved for use by the watchdog.\r\n\r\n"
      "Then, use the slider to configure the timeout. The maximum timeout is 6553.6sec, but that would be unwieldy in a sample.\r\n\r\n"
      "Finally, click the Set button to write all of this to the device."
   ;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormDestroy(TObject *Sender)
{
    delete LowOutputBitmap;
    delete HighOutputBitmap;
    delete InputBitmap;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::LevelImageClick(TObject *Sender)
{
    int I = ((TImage *) Sender)->Tag;
    if ( OutButton[I/8]->Down ) //If that bit is in an output byte.
    {
        DIOData = DIOData ^ (1 << I); //Toggle that bit.
        UpdateDIODisplay();
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OutButtonClick(TObject *Sender)
{
    int I = ((TSpeedButton *) Sender)->Tag;
    DIOData |= 0xFF << (I*8); //Set new output byte high, in case user made a mistake.
    UpdateDIODisplay();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::TimeoutTrackChange(TObject *Sender)
{
    double TimeoutSeconds = TimeoutTrack->Position / 10.0;
    TimeoutLabel->Caption = "Timeout: " + FloatToStr(TimeoutSeconds) + "sec";
}
//---------------------------------------------------------------------------

void __fastcall TForm1::UpdateDIODisplay(void)
{
    for ( int I = 0; I < 16; ++I )
    {
        if ( OutButton[I/8]->Down == false )
            LevelImage[I]->Picture->Assign(InputBitmap);
        else if ( DIOData & (1 << I) )
            LevelImage[I]->Picture->Assign(HighOutputBitmap);
        else
            LevelImage[I]->Picture->Assign(LowOutputBitmap)
        ;
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SetWatchdogButtonClick(TObject *Sender)
{
    TWatchdogConfig WatchdogConfig;

    WatchdogConfig.TimeoutDeciseconds = TimeoutTrack->Position;

    WatchdogConfig.OutMask = 0;
    for ( int I = 0; I < 2; ++I )
        if ( OutButton[I]->Down )
            WatchdogConfig.OutMask |= (1 << I)
    ;

    WatchdogConfig.DIOData = DIOData;

    unsigned long Status = GenericVendorWrite(DeviceIndex, 0x2E, 0, 0, sizeof(WatchdogConfig), &WatchdogConfig);

    if ( Status == ERROR_SUCCESS )
        BlurbMemo->Lines->Text = "Watchdog config set!";
    else
        BlurbMemo->Lines->Text = "Error " + IntToStr(Status) + " setting watchdog config."
    ;
}
//---------------------------------------------------------------------------

